/***************************************************************************
 *   Copyright (C) 2005-2009 by Hannes Hauswedell                          *
 *   kports@soulrebel.in-berlin.de                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#ifndef __BSDPORTSEARCH_H__
#define __BSDPORTSEARCH_H__


#include <QThread>
#include <QTimer>
#include <QString>

#include <math.h>

#include "bsdportglobal.h"

/* This is class for searching you ports.
   it receives search term and flags on creation. It then continously posts 
   requests for information to ports in the collection.
   Upon receving answer it filters the answer for the term and emits a signal
   if its found a port */


class QBsdPortSearch : public QObject
{
Q_OBJECT
private:
    qint64  m_flags;
    QString m_term;
    QTimer  *timer;
    uint    m_portIndex;
    
    int     delay;
    int     maxDelay;

    bool hasAvSrcPkg,
         hasAvBinPkg,
         hasInstPkg;

    int percent,
        percentbuf;
    int flagMin,
        flagMax;


    QPortList   ret;

    bool running;


    void getStringAttr(QBsdPortPkg* pkg, bool run, const QString& fu)
    {
        if (!run)
            return;
        if (!connect(pkg,   QString("2" + fu + "Signal(const QString)").toLatin1(),
                this,       SLOT(receiveString(const QString&)),
                Qt::DirectConnection))
             qDebug("could not establish connection");
             
        if (!QMetaObject::invokeMethod(pkg,
                                   QString(fu + "Slot").toLatin1(),
                                   Qt::DirectConnection))
            qDebug("could not invoke method: " + pkg->nv().toLatin1());
            
        ++delay;
    }

    void getStringListAttr(QBsdPortPkg* pkg, bool run, const QString& fu)
    {
        if (!run)
            return;
        if (!connect(pkg,   QString("2" + fu + "Signal(const QStringList)").toLatin1(),
                this,       SLOT(receiveStringList(const QStringList&)),
                Qt::DirectConnection))
            qDebug("could not establish connection");
            
        if (!QMetaObject::invokeMethod(pkg,
                                  QString(fu + "Slot").toLatin1(),
                                   Qt::DirectConnection))
            qDebug("could not invoke method: " + pkg->nv().toLatin1());
        ++delay;
    }   

public:

    QBsdPortSearch(const QString& term,
                   qint64 flags)
    {
        m_term      = term;
        m_flags     = flags;
        m_portIndex = 0;
        delay       = 0;
        maxDelay    = 10;

        percent     = 0;
        percentbuf  = 0;
        running     = false;
        timer       = 0;

        if ((!flags) || term.isEmpty())
            done();

        
        // reduce the flag-range at the start to save iterations per timeout
        flagMin     = 0;
        flagMax     = 0;
        bool minFound = false;
        for (int j = 0; j < 20; ++j)
        {
            if (m_flags & (qint64)pow(2,j))
            {
                if (!minFound)
                {
                    minFound = true;
                    flagMin = j;
                }
                if (flagMax < j)
                    flagMax = j;
            }
        }
//         qDebug("minMask:" + QString::number((qint64)pow(2,flagMin)).toLatin1());
//         qDebug("maxMask:" + QString::number((qint64)pow(2,flagMax)).toLatin1());
    }

    ~QBsdPortSearch()
    {
        delete timer;
    }
    
    void done()
    {
        if (timer)
            timer->stop();
        searchFinished();
        disconnect(this);
    }

public slots:

    void search()
    {
        if (running)
            return;
            
        running = true;
        
        timer = new QTimer(this);
        timer->setInterval(0);
        connect(timer,  SIGNAL(timeout()),
                this,   SLOT(timeout()));
                
        timer->start();
        emit searching();
    }

    void interrupt()
    {
        done();
    }

    void receiveString(const QString& s)
    {
        --delay;
        QBsdPort* p = ((QBsdPortPkg*)sender())->parent();
        if (!ret.contains(p))
            if (s.contains(m_term, Qt::CaseInsensitive))
            {
                ret.append(p);
                emit searchResult(p);
            }
    }
    void receiveStringList(const QStringList& s)
    {
        --delay;
        QBsdPort* p = ((QBsdPortPkg*)sender())->parent();
        if (!ret.contains(p))
            if (s.filter(m_term, Qt::CaseInsensitive).size() > 0 )
            {
                ret.append(p);
                emit searchResult(p);
            }
    }


    

    void timeout()
    {
        if (m_portIndex == QBsdPort::global->collection().size()-1)
        {
            done();
            return;
        }
        
        if (delay >= maxDelay)
            return;
            
        ++m_portIndex;
        
        percentbuf = m_portIndex*100/QBsdPort::global->collection().size();
        if ( percentbuf != percent)
        {
            percent = percentbuf;
            emit searchProgress(percent);
        }
        
        QBsdPort *p = QBsdPort::global->collection().at(m_portIndex);
        hasAvSrcPkg = p->hasAvSrc();
        hasAvBinPkg = p->hasAvBin();
        hasInstPkg  = p->hasInst();

        int j = flagMin;
        while (j <= flagMax)
        {
            if (m_flags & (qint64)pow(2,j))
                switch(j)
                {
                    case  0:
                        getStringAttr(p->avSrc(), hasAvSrcPkg, "name");
                        break;
                    case  1:
                        getStringAttr(p->avSrc(), hasAvSrcPkg, "comment");
                        break;
                    case  2:
                        getStringListAttr(p->avSrc(), hasAvSrcPkg, "files");
                        break;
                    case  3:
                        getStringAttr(p->avSrc(), hasAvSrcPkg, "descr");
                        break;
                    case  8:
                        getStringAttr(p->avBin(), hasAvBinPkg, "name");
                        break;
                    case  9:
                        getStringAttr(p->avBin(), hasAvBinPkg, "comment");
                        break;
                    case 16:
                        getStringAttr(p->inst(), hasInstPkg, "name");
                        break;
                    case 17:
                        getStringAttr(p->inst(), hasInstPkg, "comment");
                        break;
                    case 18:
                        getStringListAttr(p->inst(), hasInstPkg, "files");
                        break;
                    case 19:
                        getStringAttr(p->inst(), hasInstPkg, "descr");
                        break;
                }
            ++j;

        }
        
    }

    
signals:
    void    searching();
    void    searchProgress(int);
    void    searchResult(QBsdPort*);
    void    searchFinished();
};

#endif //not __BSDPORTSEARCH_H__