/***************************************************************************
 *   Copyright (C) 2005-2009 by Hannes Hauswedell                          *
 *   kports@soulrebel.in-berlin.de                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

/** Some not-so-complex widgets are defined in this file **/


#ifndef EXTRAWIDGETS_H
#define EXTRAWIDGETS_H


#include <QTimer>
#include <QString>

#include <QtGui/QLabel>
#include <QtGui/QWidget>
#include <QtGui/QStackedWidget>
#include <QtGui/QLineEdit>
#include <QtGui/QPushButton>
#include <QtGui/QHBoxLayout>
#include <QtGui/QFileDialog>

#include "bsdportglobal.h"
#include "kportsicons.h"


/// convenience class for creating hyperlinks
class QLinkLabel : public QLabel
{
Q_OBJECT
public:
    QLinkLabel( QWidget *parent = 0, Qt::WindowFlags f = 0 ) 
        : QLabel(parent, f)
    {
        setOpenExternalLinks(true);
        setTextInteractionFlags( Qt::LinksAccessibleByKeyboard 
                               | Qt::LinksAccessibleByMouse);

    }

    QLinkLabel ( const QString& text, QWidget *parent = 0, Qt::WindowFlags f = 0 )
        : QLabel(parent, f)
    {
        setOpenExternalLinks(true);
        setTextInteractionFlags( Qt::LinksAccessibleByKeyboard 
                               | Qt::LinksAccessibleByMouse);
        setLink(text);
    }

public slots:
    void    setLink(const QString& string)
    {
        QString s = string;
        if (s.endsWith("/")) /// work around stupid qt-BUG
            s.remove(-1,1);
        if ((s.contains('@')) && (!s.contains(":/")))
             setText("<a href=mailto:" + s + ">" + s + "</a>");
        else
             setText("<a href=" + s + ">" + s + "</a>");
    }

};


class QMyStackedWidget : public QStackedWidget
{
Q_OBJECT

public:

    QMyStackedWidget(QWidget* parent) : QStackedWidget(parent) {}

    QSize sizeHint() const
    {
        if (currentWidget() && currentWidget()->layout())
            return currentWidget()->sizeHint();
        return QSize();
    }

};


class QWidgetFader : public QObject
{
Q_OBJECT
private:
    QWidget     *widget;
public:
    QTimer      *fadeTimer;

    
    QWidgetFader(QWidget *w)
    {
        widget = w;
        fadeTimer = new QTimer(this);
        fadeTimer->setInterval(40);
    }


public slots:
    void setWidgetVisible(bool show)
    {
        fadeTimer->disconnect(SIGNAL(timeout()));
        if (!fadeTimer->isActive())
        {
            fadeTimer->start();
            emit beginningToFade();
        }
        connect(fadeTimer, SIGNAL(timeout()),
                this,      (show ? SLOT(grow()) : SLOT(shrink())) );
    }

    void shrink()
    {
        if (widget->size().height() > 12)
            widget->setFixedHeight(widget->size().height() - 12);
        else
        {
            fadeTimer->stop();
            hideWidget();
        }
    }

    void grow()
    {
        if (!widget->isVisible())
            widget->show();
        if (widget->size().height() < widget->sizeHint().height() - 10)
            widget->setFixedHeight(widget->size().height() + 10);
        else
        {
            fadeTimer->stop();
            showWidget();
        }
    }    

    void hideWidget()
    {
        widget->setFixedHeight(0);
        widget->hide();
        emit fadeCompleted(false);
    }

    void showWidget()
    {
        widget->setFixedHeight(widget->sizeHint().height());
        widget->show();
        emit fadeCompleted(true);
    }
signals:
    void beginningToFade();
    void fadeCompleted(bool isVisible);
};



class QFileSelecter : public QWidget
{
Q_OBJECT
public:
    QLineEdit   *edit;
    QPushButton *button;
    QHBoxLayout *layout;
    QString     caption;


    QFileSelecter(QWidget *parent = 0)
        : QWidget(parent)
    {
        setupUi();
        retranslateUi();
        caption = tr("Open File");
    }

    void setupUi()
    {
        edit        = new QLineEdit(this);
        edit->setReadOnly(true);
        
        button      = new QPushButton(this);
        connect(button, SIGNAL(clicked()),
                this,   SLOT(buttonClicked()));
                
        layout      = new QHBoxLayout(this);
        layout->addWidget(edit);
        layout->addWidget(button);
    }

    void retranslateUi()
    {
        button->setText(tr("..."));
    }

public slots:
    virtual void buttonClicked()
    {
        QString newPath = QFileDialog::getOpenFileName(this,
                                                       caption,
                                                       edit->text());
        if (!newPath.isEmpty())
            edit->setText(newPath);
    }
};

class QDirSelecter : public QFileSelecter
{
Q_OBJECT
public:
    QDirSelecter(QWidget *parent = 0)
        : QFileSelecter(parent)
    {
        caption = tr("Open Directory");
    }

public slots:
    void buttonClicked()
    {
        QString newPath = QFileDialog::getExistingDirectory(this,
                                                            caption,
                                                            edit->text());
        if (!newPath.isEmpty())
            edit->setText(newPath);
    }
};

/** A QLabel that slows textchanges down, so that every text is visible for
at least 0.5s **/
class QSlowLabel : public QLabel
{
Q_OBJECT
private:
    QStringList textQueue;
    QTimer      *timer;
    bool        freeze;

    void init()
    {
        freeze = false;
        timer = new QTimer(this);
        timer->setInterval(500);
        connect(timer,  SIGNAL(timeout()),
                this,   SLOT(timedOut()));

    }
public:
    QSlowLabel(QWidget * parent = 0, Qt::WindowFlags f = 0 )
        : QLabel(parent, f)
    {
        init();
    }
    QSlowLabel(const QString& text = QString())
        : QLabel(text)
    {
        init();
    }

    ~QSlowLabel()
    {
        delete timer;
    }
public slots:
    void    setText(const QString& text)
    {
        if (freeze)
            textQueue.append(text);
        else
        {
            freeze = true;
            QLabel::setText(text);
            timer->start();
        }
    }

    void    timedOut()
    {
        if (textQueue.count() > 0)
        {
            QLabel::setText(textQueue.at(0));
            textQueue.removeAt(0);
            update();
        }
        if (textQueue.count() == 0)
        {
            freeze = false;
            timer->stop();
        }
    }

};




/*
class QInfoPageScrollArea : public QScrollArea
{
Q_OBJECT
public:

    QVBoxLayout *widgetLayout;


    QInfoPageWidget *nameItem;
    QInfoPageWidget *webItem;




    QInfoPageScrollArea(QWidget* parent)
        : QScrollArea(parent)
    {
        setWidget(new QWidget(this));

        widgetLayout = new QVBoxLayout(widget());
        widgetLayout->setContentsMargins(0,0,0,0);

        setupNameItem();
        setupWebItem();
        // ...

        // last
        QSpacerItem* spacerItem = 
          new QSpacerItem(1, 1, QSizePolicy::Minimum, QSizePolicy::Expanding);
        widgetLayout->addItem(spacerItem);

        connectPort(0);
    }

    void    setupNameItem()
    {
        nameItem = new QInfoPageWidget(widget());

        //TODO replace with some cool button
        nameWidget->iconLabel->setPixmap(QPixmap(QString::fromUtf8(":/images/hd.png")).scaled(16,16));
    
        //TODO replace with BUTTOM icon
        nameWidget->detailsButton->setIcon(QIcon(QString::fromUtf8(":/images/hd.png")));



        widgetLayout->addWidget(nameWidget);
    }





    void    connectPort(QBsdPort *port)
    {

    }
};*/




#endif
